#define _GNU_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <memory.h>
#include <sodium.h>
#include <string.h>

#include "cpucycles.h"

int main(int argc, char *argv[]) {
	int i, iterations;
	cc_set *cc_generate;

	if (argc != 2) {
		printf("Missing iteration parameter\n");
		exit(EXIT_FAILURE);
	}

	errno = 0;
	iterations = strtol(argv[1], NULL, 10);

	if (iterations <= 0 || errno != 0 ) {
		printf("Invalid Parameter");
		if (errno != 0)
			printf(": %s", strerror(errno));
		printf("\n");
		exit(EXIT_FAILURE);
	}

	#if USE_PERF
		perf_init();
	#endif

	cc_generate = cc_set_init(iterations);

	unsigned char server[] = "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA";
	unsigned char client[] = "BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB";
	unsigned char shared[crypto_generichash_BYTES];
	
	close(open("raw_dh.csv", O_TRUNC | O_WRONLY));
	close(open("median_dh.csv", O_TRUNC | O_WRONLY));


	for(i = 0; i < iterations; i++) {
		uint64_t t = cpucycles();
		crypto_scalarmult(shared, client, server);
		cc_generate->array[i] =  cpucycles() - t;

	}

	FILE *raw_data = fopen("raw_dh.csv", "a");
	for(i = 0; i < iterations; i++) {
		fprintf(raw_data, "%llu\n", cc_generate->array[i]);
	}
	fclose(raw_data);

	FILE *median_data = fopen("median_dh.csv", "a");
	fprintf(median_data, "%llu\n", median(cc_generate));
	fclose(median_data);

	#if USE_PERF
		perf_fin();
	#endif

	return EXIT_SUCCESS;
}
