#define _GNU_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <memory.h>
#include <string.h>
#include <sodium.h>

#include "cpucycles.h"
#include "minimalt_puzzle.h"

int main(int argc, char *argv[]) {
	int i, bits, iterations;
	bool status;
	Puzzle *puzzle = malloc(sizeof(Puzzle));
	cc_set *cc_generate, *cc_solve, *cc_verify, *cc_verify_invalid;


	if (argc != 2) {
		printf("Missing iteration parameter\n");
		exit(EXIT_FAILURE);
	}

	errno = 0;
	iterations = strtol(argv[1], NULL, 10);

	if (iterations <= 0 || errno != 0 ) {
		printf("Invalid Parameter");
		if (errno != 0)
			printf(": %s", strerror(errno));
		printf("\n");
		exit(EXIT_FAILURE);
	}

	#ifdef USE_PERF
		perf_init();
	#endif

	cc_generate = cc_set_init(iterations);
	cc_solve = cc_set_init(iterations);
	cc_verify = cc_set_init(iterations);
	cc_verify_invalid = cc_set_init(iterations);

	CryptoKey *server = (CryptoKey*)"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA";
	CryptoKey *client_static = (CryptoKey*)"BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB";
	CryptoKey *client = malloc(sizeof(char)*EncryptionPublicKeySize);

	puzzleInit();

	close(open("raw_minimalt_puzzle.csv", O_TRUNC | O_WRONLY));
	close(open("median_minimalt_puzzle.csv", O_TRUNC | O_WRONLY));

	for(bits = 1; bits <= 20 ; bits++) {
		memcpy(client, client_static, EncryptionPublicKeySize);

		for(i = 0; i < iterations; i++) {
			snprintf(client, EncryptionPublicKeySize, "%u", i);

			uint64_t t = cpucycles();
			status  = puzzleGenerate(client, server, bits, puzzle);
			cc_generate->array[i] =  cpucycles() - t;

			t = cpucycles();
			status = puzzleSolve(puzzle, bits);
			cc_solve->array[i] =  cpucycles() - t;

			t = cpucycles();
			status = puzzleVerify((PuzzleSolution *)puzzle, client, server);
			cc_verify->array[i] =  cpucycles() - t;

			//a random valid nonce invalid puzzle solution
			randombytes_buf(puzzle->encryptedData, sizeof(puzzle->encryptedData));
			t = cpucycles();
			status = puzzleVerify((PuzzleSolution *)puzzle, client, server);
			cc_verify_invalid->array[i] =  cpucycles() - t;

		}

		FILE *raw_data = fopen("raw_minimalt_puzzle.csv", "a");
		for(i = 0; i < iterations; i++) {
			fprintf(raw_data, "%u;%llu;%llu;%llu;%llu\n", bits, cc_generate->array[i], cc_solve->array[i], cc_verify->array[i], cc_verify_invalid->array[i]);
		}
		fclose(raw_data);

		FILE *median_data = fopen("median_minimalt_puzzle.csv", "a");
		fprintf(median_data, "%u;%llu;%llu;%llu;%llu\n", bits, median(cc_generate), median(cc_solve), median(cc_verify), median(cc_verify_invalid));
		fclose(median_data);

	}

	#ifdef USE_PERF
		perf_fin();
	#endif

	free(client);
	exit(EXIT_SUCCESS);
}
