#define _GNU_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <gmp.h>
#include <string.h>

#include "cpucycles.h"
#include "msr_jerschow_mauve.h"


int main(int argc, char *argv[]) {
	mpz_t pow, prime;
	gmp_randstate_t rstate;
	int i, iterations;
	int search_depth = 20;
	cc_set *cc_solve, *cc_verify, *cc_verify_invalid;

	if (argc != 2) {
		printf("Missing iteration parameter\n");
		exit(EXIT_FAILURE);
	}

	errno = 0;
	iterations = strtol(argv[1], NULL, 10);

	if (iterations <= 0 || errno != 0 ) {
		printf("Invalid Parameter");
		if (errno != 0)
			printf(": %s", strerror(errno));
		printf("\n");
		exit(EXIT_FAILURE);
	}

	#ifdef USE_PERF
		perf_init();
	#endif


	cc_solve = cc_set_init(iterations);
	cc_verify = cc_set_init(iterations);
	cc_verify_invalid = cc_set_init(iterations);

	mpz_init(pow);
	mpz_init(prime);
	gmp_randinit_default(rstate);

	FILE *test_set = fopen("../../test_set_msr/primes.gmp","r");

	close(open("raw_msr_jerschow_mauve.csv", O_TRUNC | O_WRONLY));
	close(open("median_msr_jerschow_mauve.csv", O_TRUNC | O_WRONLY));

	while(mpz_inp_raw(prime, test_set) != 0) {

		for(i = 0; i < iterations; i++) {
			char *service_string;
			uint64_t t = cpucycles();

			asprintf(&service_string,"ServiceString:%u", i);

			Challenger(pow, prime, service_string, search_depth);
			cc_solve->array[i] =  cpucycles() - t;

			t = cpucycles();
			Judge(pow, prime, service_string, search_depth);
			cc_verify->array[i] = cpucycles() - t;

			//invalid random
			mpz_urandomm(pow, rstate, prime);
			t = cpucycles();
			Judge(pow, prime, service_string, search_depth);
			cc_verify_invalid->array[i] = cpucycles() - t;

			free(service_string);
		}

		FILE *raw_data = fopen("raw_msr_jerschow_mauve.csv", "a");
		for(i = 0; i < iterations; i++) {
			fprintf(raw_data, "%u;%llu;%llu;%llu\n", mpz_sizeinbase(prime, 2), cc_solve->array[i], cc_verify->array[i], cc_verify_invalid->array[i]);
		}
		fclose(raw_data);

		FILE *median_data = fopen("median_msr_jerschow_mauve.csv", "a");
		fprintf(median_data, "%u;%llu;%llu;%llu\n",mpz_sizeinbase(prime, 2), median(cc_solve), median(cc_verify), median(cc_verify_invalid));
		fclose(median_data);

	}

	#ifdef USE_PERF
		perf_fin();
	#endif

	fclose(test_set);
	mpz_clear(pow);

	exit(EXIT_SUCCESS);
}
