#ifndef MINIMALT_PUZZLE_H
#define MINIMALT_PUZZLE_H

#include <stdlib.h>
#include <stdbool.h>
#include <stdint.h>


typedef uint32_t uint32;
typedef uint64_t uint64;
typedef unsigned char uchar;
typedef unsigned char  CryptoKey;

typedef uint8_t HashValue[64];

typedef struct {
	uint8_t encryptedData[80];
	uint64_t nonce;
} PuzzleSolution;


typedef struct {
	uint8_t encryptedData[80];
	uint64_t nonce;
	HashValue hash;
	uint32_t missingBits;
} Puzzle;

enum {
	SymmetricKeySize         = 32,
	HashSize                 = 64,
	EncryptionPublicKeySize	 = 32,
    PuzzleEncryptedSize	     = 2*EncryptionPublicKeySize + 16,
    PuzzleEncryptedPaddedSize= PuzzleEncryptedSize       + 16,
    PuzzleSize               = PuzzleEncryptedSize + 8 + 4 + HashSize,
    PuzzleSolutionSize       = PuzzleEncryptedSize + 8,
};

bool puzzlesNeeded; // initially zero, set if a server and load goes too high
uint puzzleBits;    // number of zero bits to be used in a puzzle

/// \brief Initializes the puzzle crypto key CONSTANT
void     puzzleInit(void);


/// Generates a puzzle challenge based on the ephemeral Keys
///
/// @param clientKey a pointer to the client public key for this tunnel
/// @param serverKey a pointer to the server public key for this tunnel
/// @param diff how many bits are razed
/// @param k is the symmetric key used for encrypting
///
/// @return true if successful
bool     puzzleGenerate(CryptoKey *clientKey, CryptoKey *serverKey, uint32 diff, Puzzle *puzzle);

/// \brief Solves a puzzle challenge and returns a solution
///
/// @param puzzle a pointer to the challenge
///
bool     puzzleSolve(Puzzle *puzzle, unsigned int maxPuzzleBits);

/// \brief Verifies a puzzle solution on the server side using server's encryption key
/// 
/// @param puzzle solution to be verified
/// @param clientKey for this tunnel
/// @param serverKey for this tunnel
///
/// @return returns true if the solution was valid, false otherwise
bool     puzzleVerify(PuzzleSolution *puzzle, CryptoKey *clientKey, CryptoKey *serverKey); 

#endif
